@@ -23,7 +23,7 @@ import unittest
 
 import sys
 import os
-from authheaders import authenticate_message, sign_message
+from authheaders import authenticate_message, sign_message, get_domain_part
 
 #import logging
 #logging.basicConfig(level=10)
@@ -48,6 +48,8 @@ class TestAuthenticateMessage(unittest.TestCase):
         self.message5 = read_test_data("test.message_sub")
         self.message6 = read_test_data("test.message_np1")
         self.message7 = read_test_data("test.message_np2")
+        self.message8 = read_test_data("testcomma.message")
+        self.message9 = read_test_data("test_nop.message")
         self.key = read_test_data("test.private")
 
     def dnsfunc(self, domain, timeout=5):
@@ -67,6 +69,7 @@ Y+vtSBczUiKERHv1yRbcaQtZFh5wtiRrN04BLUTD21MycBX5jYchHjPY/wIDAQAB""",
           "_dmarc.example.net": """v=DMARC1\; p=none\; sp=reject""",
           "_dmarc.sub.example.net": "",
           "_dmarc.example.biz": """v=DMARC1\; p=none\; sp=quarantine\; np=reject""",
+          "_dmarc.nop.example.org": """v=DMARC1\; sp=quarantine\; np=reject""",
           "_dmarc.sub.example.biz": "",
           "_dmarc.sub2.example.biz": "",
           "sub.example.biz": None,
@@ -114,11 +117,26 @@ Y+vtSBczUiKERHv1yRbcaQtZFh5wtiRrN04BLUTD21MycBX5jYchHjPY/wIDAQAB""",
         res = authenticate_message(self.message7, "example.com", prev='Authentication-Results: example.com; dkim=fail header.d=sub2.example.biz header.i=@sub2.example.biz', spf=False, dkim=False, dnsfunc=self.dnsfunc)
         self.assertEqual(res, "Authentication-Results: example.com; dkim=fail header.d=sub2.example.biz header.i=@sub2.example.biz; dmarc=fail (Used Org Domain Record) header.from=sub2.example.biz policy.dmarc=quarantine")
 
+    def test_authenticate_dmarc_no_p(self):
+        self.maxDiff = None
+        res = authenticate_message(self.message9, "example.com", prev='Authentication-Results: example.com; dkim=fail header.d=nop.example.org header.i=@nop.example.org', spf=False, dkim=False, dnsfunc=self.dnsfunc)
+        self.assertEqual(res, "Authentication-Results: example.com; dkim=fail header.d=nop.example.org header.i=@nop.example.org; dmarc=none")
+
+    def test_authenticate_dmarc_comma(self):
+        res = authenticate_message(self.message8, "example.com", spf=False, dnsfunc=self.dnsfunc)
+        self.assertEqual(res, "Authentication-Results: example.com; dkim=pass header.d=example.com header.i=@example.com; dmarc=pass (Used From Domain Record) header.from=example.com policy.dmarc=reject")
+
     def test_prev(self):
         prev = "Authentication-Results: example.com; spf=pass smtp.mailfrom=gmail.com"
         res = authenticate_message(self.message2, "example.com", prev=prev, spf=False, dmarc=False, dnsfunc=self.dnsfunc)
         self.assertEqual(res, "Authentication-Results: example.com; spf=pass smtp.mailfrom=gmail.com; dkim=pass header.d=example.com header.i=@example.com")
 
+    def test_get_domain_part(self):
+        froms_to_test = [['test@example.com', 'example.com'], [""""Test, User" <test@example.com>""", 'example.com'], ["""Test User <test@sub2.example.biz>""", 'sub2.example.biz'], ["""=?UTF-8?B?QmVkIEJhdGggJiBCZXlvbmQ=?=<BdBth&Byond@example.com>""", 'example.com'], ]
+        for body_from in froms_to_test:
+            res = get_domain_part(body_from[0])
+            self.assertEqual(res, body_from[1])
+
 class TestChainValidation(unittest.TestCase):
     def dnsfuncb(self, domain, timeout=5):
         _dns_responses = {
